@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2Connection.java,v 1.40.2.1 2007/04/16 18:31:44 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2Connection.java,v 1.47.2.2 2008/04/01 07:19:29 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -15,7 +15,6 @@ import java.sql.*;
 import java.util.*;
 import org.postgresql.core.*;
 import org.postgresql.Driver;
-import org.postgresql.PGConnection;
 import org.postgresql.PGNotification;
 import org.postgresql.fastpath.Fastpath;
 import org.postgresql.largeobject.LargeObjectManager;
@@ -394,13 +393,11 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
     {
         if (typemap != null)
         {
-            SQLData d = (SQLData) typemap.get(type);
-            if (d != null)
+            Class c = (Class) typemap.get(type);
+            if (c != null)
             {
                 // Handle the type (requires SQLInput & SQLOutput classes to be implemented)
-                if (logger.logDebug())
-                    logger.debug("getObject(String,String) with custom typemap");
-                throw org.postgresql.Driver.notImplemented(this.getClass(), "getObject(String,String)");
+                throw new PSQLException(GT.tr("Custom type maps are not supported."), PSQLState.NOT_IMPLEMENTED);
             }
         }
 
@@ -733,7 +730,7 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
         if (level == null)
             return Connection.TRANSACTION_READ_COMMITTED; // Best guess.
 
-        level = level.toUpperCase();
+        level = level.toUpperCase(Locale.US);
         if (level.indexOf("READ COMMITTED") != -1)
             return Connection.TRANSACTION_READ_COMMITTED;
         if (level.indexOf("READ UNCOMMITTED") != -1)
@@ -939,7 +936,7 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
     }
 
     public String escapeString(String str) throws SQLException {
-        return Utils.appendEscapedString(null, str,
+        return Utils.appendEscapedLiteral(null, str,
                 protoConnection.getStandardConformingStrings()).toString();
     }
 
@@ -974,6 +971,15 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
         return _typeCache.getPGType(typeName);
     }
 
+    /**
+     * Return the oid of the array type of the given base type.
+     */
+    public int getPGArrayType(String elementTypeName) throws SQLException
+    {
+        elementTypeName = _typeCache.getTypeForAlias(elementTypeName);
+        return getPGType("_" + elementTypeName);
+    }
+
     public String getJavaClass(int oid) throws SQLException
     {
         return _typeCache.getJavaClass(oid);
@@ -990,6 +996,11 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
         return _typeCache.getPGType(oid);
     }
 
+    public int getPGArrayElement (int oid) throws SQLException
+    {
+        return _typeCache.getPGArrayElement(oid);
+    }
+
     // This is a cache of the DatabaseMetaData instance for this connection
     protected java.sql.DatabaseMetaData metadata;
 
@@ -1011,6 +1022,7 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
 
     public PGNotification[] getNotifications() throws SQLException
     {
+        getQueryExecutor().processNotifies();
         // Backwards-compatibility hand-holding.
         PGNotification[] notifications = protoConnection.getNotifications();
         return (notifications.length == 0 ? null : notifications);
@@ -1076,7 +1088,7 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
     }
 
 
-    public int getSQLType(String pgTypeName)
+    public int getSQLType(String pgTypeName) throws SQLException
     {
         return _typeCache.getSQLType(pgTypeName);
     }
