@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2003-2005, PostgreSQL Global Development Group
+* Copyright (c) 2003-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2ResultSet.java,v 1.88.2.2 2007/04/16 16:36:49 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2ResultSet.java,v 1.101.2.2 2009/04/19 16:14:46 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -23,7 +23,7 @@ import java.util.Iterator;
 import java.util.StringTokenizer;
 import java.util.Vector;
 import java.util.Calendar;
-import java.util.TimeZone;
+import java.util.Locale;
 import org.postgresql.core.*;
 import org.postgresql.largeobject.*;
 import org.postgresql.util.PGobject;
@@ -44,6 +44,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     private boolean usingOID = false; // are we using the OID for the primary key?
     private Vector primaryKeys;    // list of primary keys
     private boolean singleTable = false;
+    private String onlyTable = "";
     private String tableName = null;
     private PreparedStatement updateStatement = null;
     private PreparedStatement insertStatement = null;
@@ -65,7 +66,12 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     protected int row_offset;               // Offset of row 0 in the actual resultset
     protected byte[][] this_row;      // copy of the current result row
     protected SQLWarning warnings = null; // The warning chain
-    protected boolean wasNullFlag = false; // the flag for wasNull()
+    /**
+     * True if the last obtained column value was SQL NULL as specified by
+     * {@link #wasNull}. The value is always updated by the
+     * {@link #checkResultSet} method.
+     */
+    protected boolean wasNullFlag = false;
     protected boolean onInsertRow = false;  // are we on the insert row (for JDBC2 updatable resultsets)?
 
     public byte[][] rowBuffer = null;       // updateable rowbuffer
@@ -161,7 +167,9 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
             {
                 // Fetch all results.
                 String cursorName = getString(columnIndex);
-                String fetchSql = "FETCH ALL IN \"" + cursorName + "\"";
+
+                StringBuffer sb = new StringBuffer("FETCH ALL IN ");
+                Utils.appendEscapedIdentifier(sb, cursorName);
 
                 // nb: no BEGIN triggered here. This is fine. If someone
                 // committed, and the cursor was not holdable (closing the
@@ -173,7 +181,16 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
                 //
                 // We take the scrollability from the statement, but until
                 // we have updatable cursors it must be readonly.
-                ResultSet rs = connection.execSQLQuery(fetchSql, resultsettype, ResultSet.CONCUR_READ_ONLY);
+                ResultSet rs = connection.execSQLQuery(sb.toString(), resultsettype, ResultSet.CONCUR_READ_ONLY);
+                //
+                // In long running transactions these backend cursors take up memory space
+                // we could close in rs.close(), but if the transaction is closed before the result set, then
+                // the cursor no longer exists
+
+                sb.setLength(0);
+                sb.append("CLOSE ");
+                Utils.appendEscapedIdentifier(sb, cursorName);
+                connection.execSQLUpdate(sb.toString());
                 ((AbstractJdbc2ResultSet)rs).setRefCursor(cursorName);
                 return rs;
             }
@@ -297,8 +314,6 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     public java.sql.Array getArray(int i) throws SQLException
     {
         checkResultSet( i );
-
-        wasNullFlag = (this_row[i - 1] == null);
         if (wasNullFlag)
             return null;
 
@@ -336,7 +351,6 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     public java.io.Reader getCharacterStream(int i) throws SQLException
     {
         checkResultSet( i );
-        wasNullFlag = (this_row[i - 1] == null);
         if (wasNullFlag)
             return null;
 
@@ -386,7 +400,9 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
     public java.sql.Date getDate(int i, java.util.Calendar cal) throws SQLException
     {
-        this.checkResultSet(i);
+        checkResultSet(i);
+        if (wasNullFlag)
+            return null;
 
         if (cal != null)
             cal = (Calendar)cal.clone();
@@ -397,7 +413,9 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
     public Time getTime(int i, java.util.Calendar cal) throws SQLException
     {
-        this.checkResultSet(i);
+        checkResultSet(i);
+        if (wasNullFlag)
+            return null;
 
         if (cal != null)
             cal = (Calendar)cal.clone();
@@ -408,7 +426,9 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
     public Timestamp getTimestamp(int i, java.util.Calendar cal) throws SQLException
     {
-        this.checkResultSet(i);
+        checkResultSet(i);
+        if (wasNullFlag)
+            return null;
 
         if (cal != null)
             cal = (Calendar)cal.clone();
@@ -730,13 +750,12 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         {
 
 
-            StringBuffer deleteSQL = new StringBuffer("DELETE FROM " ).append(tableName).append(" where " );
+            StringBuffer deleteSQL = new StringBuffer("DELETE FROM " ).append(onlyTable).append(tableName).append(" where " );
 
             for ( int i = 0; i < numKeys; i++ )
             {
-                deleteSQL.append("\"");
-                deleteSQL.append( ((PrimaryKey) primaryKeys.get(i)).name );
-                deleteSQL.append("\" = ?");
+                Utils.appendEscapedIdentifier(deleteSQL, ((PrimaryKey)primaryKeys.get(i)).name);
+                deleteSQL.append(" = ?");
                 if ( i < numKeys - 1 )
                 {
                     deleteSQL.append( " and " );
@@ -792,9 +811,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
             {
                 String columnName = (String) columnNames.next();
 
-                insertSQL.append("\"");
-                insertSQL.append( columnName );
-                insertSQL.append("\"");
+                Utils.appendEscapedIdentifier(insertSQL, columnName);
                 if ( i < numColumns - 1 )
                 {
                     insertSQL.append(", ");
@@ -1168,7 +1185,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
             }
 
         }
-        selectSQL.append(" from " ).append(tableName).append(" where ");
+        selectSQL.append(" from " ).append(onlyTable).append(tableName).append(" where ");
 
         int numKeys = primaryKeys.size();
 
@@ -1232,7 +1249,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         if (!doingUpdates)
             return; // No work pending.
 
-        StringBuffer updateSQL = new StringBuffer("UPDATE " + tableName + " SET  ");
+        StringBuffer updateSQL = new StringBuffer("UPDATE " + onlyTable + tableName + " SET  ");
         
         int numColumns = updateValues.size();
         Iterator columns = updateValues.keySet().iterator();
@@ -1240,9 +1257,8 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         for (int i = 0; columns.hasNext(); i++ )
         {
             String column = (String) columns.next();
-            updateSQL.append("\"");
-            updateSQL.append( column );
-            updateSQL.append("\" = ?");
+            Utils.appendEscapedIdentifier(updateSQL, column);
+            updateSQL.append(" = ?");
             
             if ( i < numColumns - 1 )
                 updateSQL.append(", ");                
@@ -1255,9 +1271,8 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         for ( int i = 0; i < numKeys; i++ )
         {                
             PrimaryKey primaryKey = ((PrimaryKey) primaryKeys.get(i));
-            updateSQL.append("\"");
-            updateSQL.append(primaryKey.name);
-            updateSQL.append("\" = ?");
+            Utils.appendEscapedIdentifier(updateSQL, primaryKey.name);
+            updateSQL.append(" = ?");
             
             if ( i < numKeys - 1 )
                 updateSQL.append(" and ");
@@ -1626,9 +1641,13 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
             name = st.nextToken();
             if ( !tableFound )
             {
-                if (name.toLowerCase().equals("from"))
+                if ("from".equalsIgnoreCase(name))
                 {
                     tableName = st.nextToken();
+                    if ("only".equalsIgnoreCase(tableName)) {
+                        tableName = st.nextToken();
+                        onlyTable = "ONLY ";
+                    }
                     tableFound = true;
                 }
             }
@@ -1703,6 +1722,20 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
                     // Should never happen?
                     break;
 
+                case Types.BINARY:
+                case Types.LONGVARBINARY:
+                case Types.VARBINARY:
+                    if (fields[columnIndex].getFormat() == Field.BINARY_FORMAT) {
+                        rowBuffer[columnIndex] = (byte[]) valueObject;
+                    } else {
+                        try {
+                            rowBuffer[columnIndex] = PGbytea.toPGString((byte[]) valueObject).getBytes("ISO-8859-1");
+                        } catch (UnsupportedEncodingException e) {
+                            throw new PSQLException(GT.tr("The JVM claims not to support the encoding: {0}", "ISO-8859-1"), PSQLState.UNEXPECTED_ERROR, e);
+                        }
+                    }
+                    break;
+ 
                 default:
                     rowBuffer[columnIndex] = (byte[]) valueObject;
                 }
@@ -1835,7 +1868,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     public void close() throws SQLException
     {
         //release resources held (memory for tuples)
-	rows = null;
+        rows = null;        
         if (cursor != null) {
             cursor.close();
             cursor = null;
@@ -1851,7 +1884,6 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     public String getString(int columnIndex) throws SQLException
     {
         checkResultSet( columnIndex );
-        wasNullFlag = (this_row[columnIndex - 1] == null);
         if (wasNullFlag)
             return null;
 
@@ -1868,6 +1900,10 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
     public boolean getBoolean(int columnIndex) throws SQLException
     {
+        checkResultSet(columnIndex);
+        if (wasNullFlag)
+            return false; // SQL NULL
+        
         return toBoolean( getString(columnIndex) );
     }
 
@@ -1876,6 +1912,10 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
     public byte getByte(int columnIndex) throws SQLException
     {
+        checkResultSet(columnIndex);
+        if (wasNullFlag)
+            return 0; // SQL NULL
+
         String s = getString(columnIndex);
 
         if (s != null )
@@ -1921,6 +1961,10 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
     public short getShort(int columnIndex) throws SQLException
     {
+        checkResultSet(columnIndex);
+        if (wasNullFlag)
+            return 0; // SQL NULL
+
         String s = getFixedString(columnIndex);
 
         if (s != null)
@@ -1959,6 +2003,10 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
     public int getInt(int columnIndex) throws SQLException
     {
+        checkResultSet(columnIndex);
+        if (wasNullFlag)
+            return 0; // SQL NULL
+
         Encoding encoding = connection.getEncoding();
         if (encoding.hasAsciiNumbers()) {
             try {
@@ -1971,6 +2019,10 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
     public long getLong(int columnIndex) throws SQLException
     {
+        checkResultSet(columnIndex);
+        if (wasNullFlag)
+            return 0; // SQL NULL
+
         Encoding encoding = connection.getEncoding();
         if (encoding.hasAsciiNumbers()) {
             try {
@@ -1981,11 +2033,18 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         return toLong( getFixedString(columnIndex) );
     }
         
+    /**
+     * A dummy exception thrown when fast byte[] to number parsing fails and
+     * no value can be returned. The exact stack trace does not matter because
+     * the exception is always caught and is not visible to users.
+     */
     private static final NumberFormatException FAST_NUMBER_FAILED = 
         new NumberFormatException();
 
     /**
-     * Optimised byte[] to number parser.
+     * Optimised byte[] to number parser.  This code does not
+     * handle null values, so the caller must do checkResultSet
+     * and handle null values prior to calling this function.
      * 
      * @param columnIndex The column to parse.
      * @return The parsed number.
@@ -1996,16 +2055,8 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
      */
     private long getFastLong(int columnIndex) throws SQLException,
         NumberFormatException {
-
-        checkResultSet( columnIndex );
         
-        columnIndex--;
-        wasNullFlag = (this_row[columnIndex] == null);
-        if (wasNullFlag) {
-            return 0; // SQL NULL
-        }
-        
-        byte[] bytes = this_row[columnIndex];
+        byte[] bytes = this_row[columnIndex - 1];
         
         if (bytes.length == 0) {
             throw FAST_NUMBER_FAILED;
@@ -2017,7 +2068,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         if (bytes[0] == '-') {
             neg = true;
             start = 1;
-            if (bytes.length > 19) {
+            if (bytes.length == 1 || bytes.length > 19) {
                 throw FAST_NUMBER_FAILED;
             }
         } else {
@@ -2046,25 +2097,21 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     }
 
     /**
-     * Optimised byte[] to number parser.
+     * Optimised byte[] to number parser.  This code does not
+     * handle null values, so the caller must do checkResultSet
+     * and handle null values prior to calling this function.
      * 
      * @param columnIndex The column to parse.
      * @return The parsed number.
      * @throws SQLException If an error occurs while fetching column.
      * @throws NumberFormatException If the number is invalid or the
      * out of range for fast parsing. The value must then be parsed by
-     * {@link #toLong(String)}.
+     * {@link #toInt(String)}.
      */
-    private int getFastInt(int columnIndex) throws SQLException {
-        checkResultSet( columnIndex );
+    private int getFastInt(int columnIndex) throws SQLException,
+        NumberFormatException {
         
-        columnIndex--;
-        wasNullFlag = (this_row[columnIndex] == null);
-        if (wasNullFlag) {
-            return 0; // SQL NULL
-        }
-        
-        byte[] bytes = this_row[columnIndex];
+        byte[] bytes = this_row[columnIndex - 1];
         
         if (bytes.length == 0) {
             throw FAST_NUMBER_FAILED;
@@ -2076,7 +2123,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         if (bytes[0] == '-') {
             neg = true;
             start = 1;
-            if (bytes.length > 10) {
+            if (bytes.length == 1 || bytes.length > 10) {
                 throw FAST_NUMBER_FAILED;
             }
         } else {
@@ -2106,16 +2153,28 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
     public float getFloat(int columnIndex) throws SQLException
     {
+        checkResultSet(columnIndex);
+        if (wasNullFlag)
+            return 0; // SQL NULL
+
         return toFloat( getFixedString(columnIndex) );
     }
 
     public double getDouble(int columnIndex) throws SQLException
     {
+        checkResultSet(columnIndex);
+        if (wasNullFlag)
+            return 0; // SQL NULL
+
         return toDouble( getFixedString(columnIndex) );
     }
 
     public BigDecimal getBigDecimal(int columnIndex, int scale) throws SQLException
     {
+        checkResultSet(columnIndex);
+        if (wasNullFlag)
+            return null;
+        
         return toBigDecimal( getFixedString(columnIndex), scale );
     }
 
@@ -2137,45 +2196,43 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     public byte[] getBytes(int columnIndex) throws SQLException
     {
         checkResultSet( columnIndex );
-        wasNullFlag = (this_row[columnIndex - 1] == null);
-        if (!wasNullFlag)
+        if (wasNullFlag)
+            return null;
+
+        if (fields[columnIndex - 1].getFormat() == Field.BINARY_FORMAT)
         {
-            if (fields[columnIndex - 1].getFormat() == Field.BINARY_FORMAT)
+            //If the data is already binary then just return it
+            return this_row[columnIndex - 1];
+        }
+        else if (connection.haveMinimumCompatibleVersion("7.2"))
+        {
+            //Version 7.2 supports the bytea datatype for byte arrays
+            if (fields[columnIndex - 1].getOID() == Oid.BYTEA)
             {
-                //If the data is already binary then just return it
-                return this_row[columnIndex - 1];
-            }
-            else if (connection.haveMinimumCompatibleVersion("7.2"))
-            {
-                //Version 7.2 supports the bytea datatype for byte arrays
-                if (fields[columnIndex - 1].getOID() == Oid.BYTEA)
-                {
-                    return trimBytes(columnIndex, PGbytea.toBytes(this_row[columnIndex - 1]));
-                }
-                else
-                {
-                    return trimBytes(columnIndex, this_row[columnIndex - 1]);
-                }
+                return trimBytes(columnIndex, PGbytea.toBytes(this_row[columnIndex - 1]));
             }
             else
             {
-                //Version 7.1 and earlier supports LargeObjects for byte arrays
-                // Handle OID's as BLOBS
-                if ( fields[columnIndex - 1].getOID() == 26)
-                {
-                    LargeObjectManager lom = connection.getLargeObjectAPI();
-                    LargeObject lob = lom.open(getLong(columnIndex));
-                    byte buf[] = lob.read(lob.size());
-                    lob.close();
-                    return trimBytes(columnIndex, buf);
-                }
-                else
-                {
-                    return trimBytes(columnIndex, this_row[columnIndex - 1]);
-                }
+                return trimBytes(columnIndex, this_row[columnIndex - 1]);
+            }
+        }
+        else
+        {
+            //Version 7.1 and earlier supports LargeObjects for byte arrays
+            // Handle OID's as BLOBS
+            if ( fields[columnIndex - 1].getOID() == Oid.OID)
+            {
+                LargeObjectManager lom = connection.getLargeObjectAPI();
+                LargeObject lob = lom.open(getLong(columnIndex));
+                byte buf[] = lob.read(lob.size());
+                lob.close();
+                return trimBytes(columnIndex, buf);
+            }
+            else
+            {
+                return trimBytes(columnIndex, this_row[columnIndex - 1]);
             }
         }
-        return null;
     }
 
     public java.sql.Date getDate(int columnIndex) throws SQLException
@@ -2196,7 +2253,6 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     public InputStream getAsciiStream(int columnIndex) throws SQLException
     {
         checkResultSet( columnIndex );
-        wasNullFlag = (this_row[columnIndex - 1] == null);
         if (wasNullFlag)
             return null;
 
@@ -2227,7 +2283,6 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     public InputStream getUnicodeStream(int columnIndex) throws SQLException
     {
         checkResultSet( columnIndex );
-        wasNullFlag = (this_row[columnIndex - 1] == null);
         if (wasNullFlag)
             return null;
 
@@ -2258,7 +2313,6 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     public InputStream getBinaryStream(int columnIndex) throws SQLException
     {
         checkResultSet( columnIndex );
-        wasNullFlag = (this_row[columnIndex - 1] == null);
         if (wasNullFlag)
             return null;
 
@@ -2277,7 +2331,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         else
         {
             // In 7.1 Handle as BLOBS so return the LargeObject input stream
-            if ( fields[columnIndex - 1].getOID() == 26)
+            if ( fields[columnIndex - 1].getOID() == Oid.OID)
             {
                 LargeObjectManager lom = connection.getLargeObjectAPI();
                 LargeObject lob = lom.open(getLong(columnIndex));
@@ -2413,8 +2467,6 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         Field field;
 
         checkResultSet(columnIndex);
-
-        wasNullFlag = (this_row[columnIndex - 1] == null);
         if (wasNullFlag)
             return null;
 
@@ -2458,9 +2510,12 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         if (columnNameIndexMap == null)
         {
             columnNameIndexMap = new HashMap(fields.length * 2);
-            for (int i = 0; i < fields.length; i++)
+            // The JDBC spec says when you have duplicate columns names,
+            // the first one should be returned.  So load the map in
+            // reverse order so the first ones will overwrite later ones.
+            for (int i = fields.length - 1; i >= 0; i--)
             {
-                columnNameIndexMap.put(fields[i].getColumnLabel().toLowerCase(), new Integer(i + 1));
+                columnNameIndexMap.put(fields[i].getColumnLabel().toLowerCase(Locale.US), new Integer(i + 1));
             }
         }
 
@@ -2470,7 +2525,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
             return index.intValue();
         }
 
-        index = (Integer)columnNameIndexMap.get(columnName.toLowerCase());
+        index = (Integer)columnNameIndexMap.get(columnName.toLowerCase(Locale.US));
         if (index != null)
         {
             columnNameIndexMap.put(columnName, index);
@@ -2497,12 +2552,9 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
      */
     public String getFixedString(int col) throws SQLException
     {
-        // Handle SQL Null
-        wasNullFlag = (this_row[col - 1] == null);
-        if (wasNullFlag)
-            return null;
-
         String s = getString(col);
+        if (s == null)
+            return null;
 
         // if we don't have at least 2 characters it can't be money.
         if (s.length() < 2)
@@ -2569,6 +2621,14 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
             throw new PSQLException(GT.tr("The column index is out of range: {0}, number of columns: {1}.", new Object[]{new Integer(column), new Integer(fields.length)}), PSQLState.INVALID_PARAMETER_VALUE );
     }
 
+    /**
+     * Checks that the result set is not closed, it's positioned on a
+     * valid row and that the given column number is valid. Also
+     * updates the {@link #wasNullFlag} to correct value.
+     * 
+     * @param column The column number to check. Range starts from 1.
+     * @throws SQLException If state or column is invalid.
+     */
     protected void checkResultSet( int column ) throws SQLException
     {
         checkClosed();
@@ -2576,6 +2636,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
             throw new PSQLException(GT.tr("ResultSet not positioned properly, perhaps you need to call next."),
                                     PSQLState.INVALID_CURSOR_STATE);
         checkColumnIndex(column);
+        wasNullFlag = (this_row[column - 1] == null);
     }
 
     //----------------- Formatting Methods -------------------
@@ -2832,7 +2893,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     // values but retains column type info.
     //
 
-    class NullObject extends PGobject {
+    static class NullObject extends PGobject {
         NullObject(String type) {
             setType(type);
         }
